<?php
/*************************************
 * SPDX-FileCopyrightText: 2009-2020 Vtenext S.r.l. <info@vtenext.com> 
 * SPDX-License-Identifier: AGPL-3.0-only  
 ************************************/

class CustomerPortal {
	
	var $defaultPortalModules = array("HelpDesk","Faq","Invoice","Quotes","Products","Services","Documents","Contacts","Accounts","Assets","ProjectMilestone","ProjectTask","ProjectPlan",);	//crmv@16644	//crmv@16858
	
	/**
	 * Invoked when special actions are performed on the module.
	 * @param String Module name
	 * @param String Event Type
	 */
	function vtlib_handler($moduleName, $eventType) {
		
		require_once('include/utils/utils.php');
		global $adb,$mod_strings,$table_prefix;
		
		if($eventType == 'module.postinstall') {
			$portalmodules = $this->defaultPortalModules;
			
			$res = $adb->query("SELECT max(sequence) AS max_tabseq FROM ".$table_prefix."_customerportal_tabs");
			$tabseq = $adb->query_result($res,0,'max_tabseq');
			$i = ++$tabseq;
			foreach($portalmodules as $module) {
				++$i;
				$this->addModule($module,$i);
			}
			$adb->query("INSERT INTO ".$table_prefix."_customerportal_prefs(tabid,prefkey,prefvalue) VALUES (0,'userid',1)");
			
			// Mark the module as Standard module
			$adb->pquery('UPDATE '.$table_prefix.'_tab SET customized=0 WHERE name=?', array($moduleName));
			
			$fieldid = $adb->getUniqueID($table_prefix.'_settings_field');
			$blockid = getSettingsBlockId('LBL_COMMUNICATION_TEMPLATES'); // crmv@98866
			$seq_res = $adb->pquery("SELECT max(sequence) AS max_seq FROM ".$table_prefix."_settings_field WHERE blockid = ?", array($blockid));
			if ($adb->num_rows($seq_res) > 0) {
				$cur_seq = $adb->query_result($seq_res, 0, 'max_seq');
				if ($cur_seq != null)	$seq = $cur_seq + 1;
			}
			
			$adb->pquery('INSERT INTO '.$table_prefix.'_settings_field(fieldid, blockid, name, iconpath, description, linkto, sequence)
				VALUES (?,?,?,?,?,?,?)', array($fieldid, $blockid, 'LBL_CUSTOMER_PORTAL', 'portal_icon.png', 'PORTAL_EXTENSION_DESCRIPTION', 'index.php?module=CustomerPortal&action=index&parenttab=Settings', $seq));
			
			
		} else if($eventType == 'module.disabled') {
			// TODO Handle actions when this module is disabled.
		} else if($eventType == 'module.enabled') {
			// TODO Handle actions when this module is enabled.
		} else if($eventType == 'module.preuninstall') {
			// TODO Handle actions when this module is about to be deleted.
		} else if($eventType == 'module.preupdate') {
			// TODO Handle actions before this module is updated.
		} else if($eventType == 'module.postupdate') {
			// TODO Handle actions after this module is updated.
		}
	}
	
	function addModule($module, $seq=null) {
		global $adb, $table_prefix;
		if (empty($tabseq)) {
			$res = $adb->query("SELECT max(sequence) AS max_tabseq FROM ".$table_prefix."_customerportal_tabs");
			$tabseq = $adb->query_result($res,0,'max_tabseq');
			$seq = ++$tabseq;
		}
		$tabid = getTabid($module);
		if ($tabid) {
			$adb->query("INSERT INTO ".$table_prefix."_customerportal_tabs (tabid,visible,sequence) VALUES ($tabid,1,$seq)");
			$adb->query("INSERT INTO ".$table_prefix."_customerportal_prefs(tabid,prefkey,prefvalue) VALUES ($tabid,'showrelatedinfo',1)");
		}
	}
}