<?php
/*************************************
 * SPDX-FileCopyrightText: 2009-2020 Vtenext S.r.l. <info@vtenext.com> 
 * SPDX-License-Identifier: AGPL-3.0-only  
 ************************************/

require_once('include/utils/utils.php');

/* Function to get a list of modules that is supporting Customer Portal
 */
function cp_getPortalModuleinfo(){
	global $adb,$table_prefix;
	
	$query = $adb->query("SELECT ".$table_prefix."_customerportal_tabs.*,".$table_prefix."_tab.name from ".$table_prefix."_customerportal_tabs
							INNER JOIN ".$table_prefix."_tab ON ".$table_prefix."_customerportal_tabs.tabid = ".$table_prefix."_tab.tabid and ".$table_prefix."_tab.presence = 0
							ORDER BY ".$table_prefix."_customerportal_tabs.sequence");
	$rows = $adb->num_rows($query);
	for($i = 0;$i < $rows; $i++){
		$portalmodules[$i]['tabid']  = $adb->query_result($query,$i,'tabid');
		$portalmodules[$i]['visible']  = $adb->query_result($query,$i,'visible');
		$portalmodules[$i]['sequence'] = $adb->query_result($query,$i,'sequence');
		$portalmodules[$i]['name'] = $adb->query_result($query,$i,'name');
	}
	return $portalmodules;
}

/* Function is used to change the order of modules in Customer portal
 *
 */
function cp_changeTabOrder($tabid,$move) {
	global $adb,$log,$table_prefix;
	$log->fatal("$tabid,$move");
	if($move == 'Down'){
		$sequence = $adb->pquery("SELECT sequence FROM ".$table_prefix."_customerportal_tabs WHERE tabid = ?", array($tabid));
		$oldsequence = $adb->query_result($sequence,0,'sequence');
		$query = "SELECT sequence,tabid FROM ".$table_prefix."_customerportal_tabs WHERE sequence > ? ORDER BY SEQUENCE";
		$params = Array($oldsequence);
		$nexttab = $adb->limitpQuery($query,0,1,$params);
		$newsequence = $adb->query_result($nexttab,0,'sequence');
		$righttabid = $adb->query_result($nexttab,0,'tabid');
		
		$adb->pquery("UPDATE ".$table_prefix."_customerportal_tabs set sequence=? WHERE tabid = ?", array($newsequence, $tabid));
		$adb->pquery("UPDATE ".$table_prefix."_customerportal_tabs set sequence=? WHERE tabid = ?", array($oldsequence, $righttabid));
	}elseif ($move == 'Up') {
		$sequence = $adb->pquery("SELECT sequence FROM ".$table_prefix."_customerportal_tabs WHERE tabid = ?", array($tabid));
		$oldsequence = $adb->query_result($sequence,0,'sequence');
		$query = "SELECT sequence,tabid FROM ".$table_prefix."_customerportal_tabs WHERE sequence < ? ORDER BY SEQUENCE DESC";
		$params = Array($oldsequence);
		$nexttab = $adb->limitpQuery($query,0,1,$params);
		$newsequence = $adb->query_result($nexttab,0,'sequence');
		$lefttabid = $adb->query_result($nexttab,0,'tabid');
		
		$adb->pquery("UPDATE ".$table_prefix."_customerportal_tabs SET sequence=? WHERE tabid = ?", array($newsequence, $tabid));
		$adb->pquery("UPDATE ".$table_prefix."_customerportal_tabs SET sequence=? WHERE tabid = ?", array($oldsequence, $lefttabid));
	}	
}

/* Function to obtain related info for Customer Portal(contacts can see other contacts info ar not)
 */

function cp_getContactsViewInfo() {
	global $adb,$table_prefix;
	$relatedinfo = $adb->query("SELECT ".$table_prefix."_customerportal_prefs.*,".$table_prefix."_tab.name FROM ".$table_prefix."_customerportal_prefs
									INNER JOIN ".$table_prefix."_tab ON ".$table_prefix."_tab.tabid = ".$table_prefix."_customerportal_prefs.tabid
									WHERE prefkey = 'showrelatedinfo' AND ".$table_prefix."_tab.presence = 0");
	$norows = $adb->num_rows($relatedinfo);
	for($i = 0;$i < $norows;$i++) {
		$module_viewall[$i]['module'] = $adb->query_result($relatedinfo,$i,'name');
		$module_viewall[$i]['value'] = $adb->query_result($relatedinfo,$i,'prefvalue');
	}
	return $module_viewall;
}

/* Function to show/hide the tabs(modules) in Customer Portal
 */
function cp_changeModuleVisibility($tabid,$status) {
	global $adb,$log,$table_prefix;
	if($status == 'module_disable'){
		$updatevisibility = $adb->pquery("UPDATE ".$table_prefix."_customerportal_tabs SET visible = 0 WHERE tabid = ?", array($tabid));
	}else {
		$updatevisibility = $adb->pquery("UPDATE ".$table_prefix."_customerportal_tabs SET visible = 1 WHERE tabid = ?", array($tabid));
	}
}

/* Function to save Advanced info fro Customer Portal
 */
function cp_saveAdvancedSettings($input) {
	global $adb,$table_prefix;
	
	$portalmodules = array();
	$portalmodulesinfo = cp_getPortalModuleinfo();
	foreach($portalmodulesinfo as $portalmoduleinfo) {
		$portalmodules[] = $portalmoduleinfo['name'];
	}
	foreach($portalmodules as $modules) {
		$view = $input['view_'.$modules];
		$tabid = getTabid($modules);
		if($view == 'showall'){
			$adb->pquery("UPDATE ".$table_prefix."_customerportal_prefs SET prefvalue = 1  WHERE prefkey = 'showrelatedinfo' and tabid = ?", array($tabid));
		}else {
			$adb->pquery("UPDATE ".$table_prefix."_customerportal_prefs SET prefvalue = 0  WHERE prefkey = 'showrelatedinfo' and tabid = ?", array($tabid));
		}
	}
	
	//user update
	$userid = $input['userid'];
	$adb->pquery("UPDATE ".$table_prefix."_customerportal_prefs SET prefvalue = ? WHERE prefkey = 'userid' and tabid = 0", array($userid));
	
	//crmv@17713
	$templateid = $input['templateid'];
	$adb->pquery("UPDATE ".$table_prefix."_notifyscheduler SET notificationbody = ? WHERE schedulednotificationid = 5", array($templateid));
	//crmv@17713e
}

/*	It gives you a list of users
 *
 */
function cp_getUsers(){
	global $adb,$table_prefix;
	$res = $adb->query("SELECT id,user_name from ".$table_prefix."_users");
	$norows = $adb->num_rows($res);
	for($i = 0;$i < $norows;$i++) {
		$users[$i]['id'] = $adb->query_result($res,$i,'id');
		$users[$i]['name'] = $adb->query_result($res,$i,'user_name');
	}
	return $users;
}

/* Function to get the customer portal user id
 */
function cp_getCurrentUser() {
	global $adb,$table_prefix;
	$res = $adb->query("SELECT prefvalue FROM ".$table_prefix."_customerportal_prefs WHERE prefkey = 'userid' AND tabid = 0");
	$userid = $adb->query_result($res,0,'prefvalue');
	if($userid != '') {
		return $userid;
	}
	return false;
}
//crmv@17713
function cp_getTemplates() {
	global $adb,$table_prefix;
	$sql = "select templateid,templatename from ".$table_prefix."_emailtemplates order by templatename";
	$result = $adb->query($sql);
	$templates = array();
	if ($result)
		while ($row = $adb->fetchByAssoc($result))
			$templates[$row['templateid']] = $row['templatename'];
	return $templates;
}
function cp_getCurrentTemplate() {
	global $adb,$table_prefix;
	$sql = "select notificationbody from ".$table_prefix."_notifyscheduler WHERE schedulednotificationid = 5";
	$result = $adb->query($sql);
	if ($result)
		return $adb->query_result($result,0,'notificationbody');
	else
		return false;
}
//crmv@17713e
?>